#eog-image-gallery-scrollbar {
	-GtkScrollbar-has-backward-stepper: false;
	-GtkScrollbar-has-secondary-backward-stepper: false;
	-GtkScrollbar-has-forward-stepper: false;
	-GtkScrollbar-has-secondary-forward-stepper: false;
}

#eog-image-gallery-scrollbar.horizontal slider {
	min-width: 60px;
}

#eog-sidebar-title {
	font-weight: bold;
}

statusbar {
	border-top: 1px solid @borders;
}

#eog-thumb-nav {
	border-top: 1px solid @borders;
}

#eog-thumb-nav scrolledwindow {
	border-bottom: none;
}

#eog-thumb-nav scrolledwindow undershoot.right {
	background: none;
	box-shadow: inset -6px 0 3px -6px @borders;
}

#eog-thumb-nav scrolledwindow undershoot.left {
	background: none;
	box-shadow: inset 6px 0 3px -6px @borders;
}

#eog-thumb-nav button {
	border-width: 1px 0 0;
	border-radius: 0;
	box-shadow: none;
}
